# -*- coding: utf-8 -*-
##############################################################################
#
#    training_ubercart_webshop module for OpenERP, Fetch ubercart orders from IFBL Website
#    Copyright (C) 2011 Thamini SàRL (<http://http://www.thamini.lu) Jean-Christophe VASSORT
#
#    This file is a part of training_ubercart_webshop
#
#    training_ubercart_webshop is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    training_ubercart_webshop is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import os
import sys
import netsvc
from tools.config import config
import pprint

class training_ubercart_webshop_subscription_lines(osv.osv):
    _name = "training.ubercart.webshop.subscription.lines"

training_ubercart_webshop_subscription_lines()

class training_ubercart_webshop_drupal_users(osv.osv):
    _name = "training.ubercart.webshop.drupal.users"
    _description = "mapping between contact's Job and drupal users"

    _columns = {
        'drupal_user_id': fields.integer('Drupal User ID', ),
        'drupal_email': fields.char('E-mail', size=128, ),
        'drupal_first_name': fields.char('First Name', size=64, ),
        'drupal_last_name': fields.char('Last Name', size=64, ),
        'drupal_company': fields.char('Company', size=64, ),
        'drupal_street1': fields.char('Street', size=256, ),
        'drupal_street2': fields.char('Street2', size=256, ),
        'drupal_city': fields.char('City', size=64, ),
        'drupal_country': fields.char('Country', size=64, ),
        'drupal_state_province': fields.char('State/Province', size=64, ),
        'drupal_postal_code': fields.char('Postal Code', size=64, ),
        'drupal_phone_number': fields.char('Phone number', size=64, ),
        'job_id': fields.many2one('res.partner.job', 'Contact\'s Job', ),
        'partner_id': fields.related('job_id', 'name', type='many2one', relation='res.partner', string="Partner", readonly = True),
        'state': fields.selection([('draft', 'Unknown'),
                                  ('to_confirm', 'To Confirm'),
                                  ('validated', 'Matched')], 'Drupal matching status', required = True, readonly = True)
    }

    _defaults = {
        'state': lambda *a: 'draft'
    }

    def name_get(self, cr, user, ids, context={}):
        if not len(ids):
            return []
        res = []
        for id in ids:
            for r in self.browse(cr, user, [id], ['job_id']):
                try:
                    obj = r.job_id.contact_id
                    res.append((id, ("%s %s") % (obj.first_name, obj.name)))
                except:
                    res.append((id, ("%s %s") % (r.drupal_first_name, r.drupal_last_name)))

        return res

    def action_workflow_draft(self, cr, uid, ids, context={}):
        w = netsvc.LocalService('workflow')
        self.pool.get('training.ubercart.webshop.orders').action_update_all_matching_confirmed_orders(cr, uid, ids, ids[0], w)
        return self.write(cr, uid, ids, {'state' : 'draft', 'job_id': ''}, context=context)

    def action_workflow_to_confirm(self, cr, uid, ids, context={}):
        return self.write(cr, uid, ids, {'state' : 'to_confirm'}, context=context)

    def action_workflow_validated(self, cr, uid, ids, context={}):
        w = netsvc.LocalService('workflow')
        self.pool.get('training.ubercart.webshop.orders').action_update_all_draft_orders(cr, uid, ids, ids[0], w)
        return self.write(cr, uid, ids, {'state' : 'validated'}, context=context)

    def on_change_job(self, cr, uid, ids, job_id, context=None):
        if job_id is False:
            return {'value': {
                'partner_id': '',
                'state': 'draft',
            }}

        else:
            for o in self.pool.get('res.partner.job').browse(cr, uid, [job_id], context):
                partner_id = o.contact_id.partner_id.id

            return {'value': {
                'partner_id': partner_id,
                'state': 'to_confirm',
            }}

    def on_change_state(self, cr, uid, ids, context={}):
        print 'on_change'

    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(('Warning'),
                             ("You can't delete Drupal users"))

        return super(training_ubercart_webshop_drupal_users, self).unlink(cr, uid, ids, context=context)

training_ubercart_webshop_drupal_users()

class training_ubercart_webshop_orders(osv.osv):
    _name = "training.ubercart.webshop.orders"
    _description = "Display ubercart orders from IFBL Website"

    def on_change_job(self, cr, uid, ids, job_id, context=None):
        if job_id is False:
            return {'value': {
                'partner_id': '',
            }}

        else:
            for o in self.browse(cr, uid, ids):
                self.pool.get('training.ubercart.webshop.drupal.users').write(cr, uid, o.drupal_user_id.id, {
                    'job_id': job_id,
                })
                drupal_user_id = self.pool.get('training.ubercart.webshop.drupal.users').name_get(cr, uid, [o.drupal_user_id.id], context)

            for o in self.pool.get('res.partner.job').browse(cr, uid, [job_id], context):
                if o.contact_id.partner_id.id is not False:
                    partner_id = o.contact_id.partner_id.id
                else:
                    partner_id = o.address_id.partner_id.id

            return {'value': {
                'drupal_user_id': drupal_user_id[0],
                'partner_id': partner_id,
            }}

    def action_update_all_draft_orders(self, cr, uid, ids, drupal_user_id, w, context=None):
        for id in self.search(cr, uid, [('state', '=', 'draft'), ('drupal_user_id', '=', drupal_user_id)]):
            w.trg_validate(uid, 'training.ubercart.webshop.orders', id, 'signal_matching_confirmed', cr)

    def action_update_all_matching_confirmed_orders(self, cr, uid, ids, drupal_user_id, w, context=None):
        for id in self.search(cr, uid, [('state', '=', 'match_confirmed'), ('drupal_user_id', '=', drupal_user_id)]):
            w.trg_validate(uid, 'training.ubercart.webshop.orders', id, 'reset_to_draft', cr)

    ##
    # Workflows methods
    ##
    
    def action_confirm_drupal_user(self, cr, uid, ids, context=None):
        w = netsvc.LocalService('workflow')
        for o in self.browse(cr, uid, ids):
            w.trg_validate(uid, 'training.ubercart.webshop.drupal.users', o.drupal_user_id.id, 'action_validate', cr)

            # update all draft orders
            self.action_update_all_draft_orders(cr, uid, ids, o.drupal_user_id.id, w)

    def action_reset_drupal_user(self, cr, uid, ids, context=None):
        w = netsvc.LocalService('workflow')
        for o in self.browse(cr, uid, ids):
            w.trg_validate(uid, 'training.ubercart.webshop.drupal.users', o.drupal_user_id.id, 'reset_to_draft', cr)
            self.write(cr, uid, ids[0], {'job_id': '',})

            # update all matching confirmed orders
            self.action_update_all_matching_confirmed_orders(cr, uid, ids, o.drupal_user_id.id, w)

    def action_workflow_matching_confirmed(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'match_confirmed'})

    def action_workflow_processing_subscription(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'process_sub'})
        sublines_pool = self.pool.get('training.ubercart.webshop.subscription.lines')
        openerp_drupal_users_pool = self.pool.get('training.ubercart.webshop.drupal.users')
        subscription_pool = self.pool.get('training.subscription')
        subscription_line_pool = self.pool.get('training.subscription.line')
        session_pool = self.pool.get('training.session')

        drupal_user_id = self.browse(cr, uid, ids)[0].drupal_user_id.id
        drupal_user = openerp_drupal_users_pool.browse(cr, uid, drupal_user_id, context = context)
        
        training_subl_obj = self.browse(cr, uid, ids)[0].subscription_lines_ids
        drupal_order_id = self.read(cr, uid, ids, ['drupal_order_id'])[0]['drupal_order_id']

        subscription_lines = []
        for o in training_subl_obj:
            subscription_lines.append({'session_id': o.session_id.id, 'exam_session_id': o.exam_session_id.id})

        sub_id = self.create_new_subscription(cr, uid, ids, context, subscription_pool, drupal_user, drupal_order_id)
        self.create_new_subscription_lines(cr, uid, ids, context, sub_id, subscription_line_pool, drupal_user, session_pool, subscription_lines)
        ##Compute Exam Subscription
        subscription_pool.action_subscription_line_compute(cr, uid, [sub_id], context)
        ##confirm subscription
        self.action_confirm_subscription_line(cr, uid, ids, context, sub_id, subscription_line_pool)
        self.action_confirm_subscription(cr, uid, ids, context, sub_id)
        ##create invoice
        self.action_create_invoice(cr, uid, sub_id, context)
        ## drupal order is done
        self.write(cr, uid, ids, {'subscription_id': sub_id})

    def action_reset_to_draft(self, cr, uid, ids, context=None):
        drupal_user_state = self.pool.get('training.ubercart.webshop.orders').read(cr, uid, ids, ['drupal_user_state'])[0]['drupal_user_state']
        w = netsvc.LocalService('workflow')
        w.trg_validate(uid, 'training.ubercart.webshop.orders', ids[0], 'reset_to_draft', cr)

        if drupal_user_state == u'validated':
            w.trg_validate(uid, 'training.ubercart.webshop.orders', ids[0], 'signal_matching_confirmed', cr)

    def action_confirm_web_subscription(self, cr, uid, ids, context=None):
        w = netsvc.LocalService('workflow')
        w.trg_validate(uid, 'training.ubercart.webshop.orders', ids[0], 'signal_confirm', cr)
        w.trg_validate(uid, 'training.ubercart.webshop.orders', ids[0], 'signal_done', cr)

    def action_workflow_error(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'error'})

    def action_workflow_done(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'done'})
    ##
    # Subscriptions methods
    ##

    def create_new_subscription(self, cr, uid, ids, context, subscription_pool, drupal_user, drupal_order_id):
        d = subscription_pool.on_change_partner(cr, uid, ids, drupal_user.job_id.name.id)
        if d.get('value'):
            d['value']['partner_id'] = drupal_user.job_id.name.id
            d['value']['is_from_web'] = True
            d['value']['custom_ref'] = 'drp-sub-' + str(drupal_order_id)
            d['value']['origin'] = 'Drupal Website'
            return subscription_pool.create(cr, uid, d['value'], context)

    def create_new_subscription_lines(self, cr, uid, ids, context, sub_id, subscription_line_pool, drupal_user, session_pool, subscription_lines):
        for d in subscription_lines:
            obj = session_pool.browse(cr, uid, d['session_id'], context=context)
            res = subscription_line_pool.on_change_job(cr, uid, ids, drupal_user.job_id.id, context)
            if res.get('value'):
                values = res['value']

                res = subscription_line_pool.on_change_session(cr, uid, ids, d['session_id'], obj.offer_id.product_line_id.price_list_id.id, drupal_user.job_id.name.id, context)
                if res.get('value'):
                    values.update(res['value'])

                    values['job_id'] = drupal_user.job_id.id
                    values['session_id'] = d['session_id']
                    values['exam_session_id'] = d['exam_session_id']
                    values['subscription_id'] = sub_id

                    subscription_line_pool.create(cr, uid, values)

    def action_confirm_subscription_line(self, cr, uid, ids, context, sub_id, subscription_line_pool):
        sl_ids = subscription_line_pool.search(cr, uid, [('subscription_id', '=', sub_id)])
        w = netsvc.LocalService('workflow')
        for id in sl_ids:
            w.trg_validate(uid, 'training.subscription.line', id, 'signal_confirm', cr)

    def action_confirm_subscription(self, cr, uid, ids, context, sub_id):
        w = netsvc.LocalService('workflow')
        w.trg_validate(uid, 'training.subscription', sub_id, 'signal_confirm', cr)

    def action_create_invoice(self, cr, uid, sub_id, context):
        subl_proxy = self.pool.get('training.subscription.line')
        subl_ids = subl_proxy.search(cr, uid, [('subscription_id','=',sub_id)])
        invoice_ids = subl_proxy.action_create_invoice(cr, uid, subl_ids, {'group_by_subscription': True})

    _columns = {
        'create_date': fields.datetime('Creation Date', readonly = True),
        'drupal_order_id': fields.integer('Drupal Order ID', readonly = True, ),

        'partner_id': fields.related('drupal_user_id', 'job_id', 'name', type='many2one', relation='res.partner', string="Partner", readonly = True),
        'job_id': fields.related('drupal_user_id', 'job_id', type='many2one', relation='res.partner.job', string="Contact Job"),
        #'contact_id': fields.related('drupal_user_id', 'contact_job', type='many2one', relation='res.partner.job', string="Contact"),
        #'address_id': fields.related('drupal_user_id', 'contact_job', 'address_id', type='many2one', relation='res.partner.address', string="Address"),
        'drupal_user_state': fields.related('drupal_user_id', 'state', type='selection', selection=[('draft', 'Unknown'),('to_confirm', 'To Confirm'),('validated', 'Matched')], string="Drupal Matching Status", readonly = True),

        'drupal_user_id': fields.many2one('training.ubercart.webshop.drupal.users', 'Drupal User', readonly = True, ),
        'drupal_first_name': fields.related('drupal_user_id', 'drupal_first_name', type='char', size=64, string = 'First name', readonly = True,),
        'drupal_last_name': fields.related('drupal_user_id', 'drupal_last_name', type='char', size=64, string = 'Last name', readonly = True,),
        'drupal_email': fields.related('drupal_user_id', 'drupal_email', type='char', size=128, string="email", readonly = True,),
        'drupal_company': fields.related('drupal_user_id', 'drupal_company', type='char', size = 64, string = "Company", readonly = True,),
        'drupal_street1': fields.related('drupal_user_id', 'drupal_street1', type='char', size = 256, string = "Street", readonly = True,),
        'drupal_street2': fields.related('drupal_user_id', 'drupal_street2', type='char', size = 256, string = "Street2", readonly = True,),
        'drupal_city': fields.related('drupal_user_id', 'drupal_city', type='char', size = 64, string = 'City', readonly = True,),
        'drupal_country': fields.related('drupal_user_id', 'drupal_country', type='char', size=64, string = 'Country', readonly = True,),
        'drupal_state_province': fields.related('drupal_user_id', 'drupal_state_province', type='char', size = 64, string = 'State/Province', readonly = True,),
        'drupal_postal_code': fields.related('drupal_user_id', 'drupal_postal_code', type = 'char', size = 64, string = 'Postal Code', readonly = True,),
        'drupal_phone_number': fields.related('drupal_user_id', 'drupal_phone_number', type = 'char', size=64, string = 'Phone number', readonly = True,),
        
        'subscription_lines_ids': fields.one2many('training.ubercart.webshop.subscription.lines', 'order_id', 'Web Subscription Lines', readonly = True),
        'drupal_messages': fields.text('Messages', readonly = True),
        'state': fields.selection([('draft', 'Draft'),
                                  ('cancelled', 'Cancelled'),
                                   ('match_confirmed', 'Matching Confirmed'),
                                   ('process_sub', 'Processing Subscription'),
                                   ('error', 'Error'),
                                  ('done', 'Done')], 'Order State', readonly = True, ),
        'subscription_id': fields.many2one('training.subscription', 'Subscription'),
    }

    _rec_name = 'drupal_order_id'
    _defaults = {
        'state': lambda *a: 'draft',
    }

    def unlink(self, cr, uid, ids, context=None):
        if self.browse(cr, uid, ids)[0].subscription_id:
            raise osv.except_osv(('Warning'),
                                 ("You can't delete this web subscription, it's linked to a subscription"))

        return super(training_ubercart_webshop_orders, self).unlink(cr, uid, ids, context=context)

training_ubercart_webshop_orders()

class training_ubercart_webshop_subscription_lines(osv.osv):
    _name = "training.ubercart.webshop.subscription.lines"
    _description = "Subscription lines from training.ubercart.webshop.orders object"

    _columns = {
        'exam_session_id': fields.many2one('training.session', 'Exam Session', domain="[('offer_id.kind', '=', 'exam')]", ),
        'session_id': fields.many2one('training.session', 'Session', domain="[('offer_id.kind', '!=', 'exam')]", ),
        'order_id': fields.many2one('training.ubercart.webshop.orders', 'Order ID', ),
    }

training_ubercart_webshop_subscription_lines()

class training_ubercart_webshop_refresh_orders(osv.osv_memory):
    _name = "training.ubercart.webshop.refresh.orders"
    _description = "Fetch and refresh ubercart orders from IFBL website"

    def refresh_orders(self, cr, uid, ids, context=None):
        orders_pool = self.pool.get('training.ubercart.webshop.orders')
        sublines_pool = self.pool.get('training.ubercart.webshop.subscription.lines')
        openerp_drupal_users_pool = self.pool.get('training.ubercart.webshop.drupal.users')
        subscription_pool = self.pool.get('training.subscription')
        subscription_line_pool = self.pool.get('training.subscription.line')
        session_pool = self.pool.get('training.session')
        w = netsvc.LocalService('workflow')

        """
        Connect to Drupal to get last orders
        """
        dpc = self.pool.get('drupal.client.config.selection').dpc(cr, uid)

        # get Drupal orders who are not present in OpenERP
        order_ids = orders_pool.search(cr, uid, [])
        drupal_orders = dpc.training.getOrders([o['drupal_order_id'] for o in orders_pool.browse(cr, uid, order_ids, context=context)])

        for order_id in drupal_orders:
            # Check if User exists in OpenERP
            if openerp_drupal_users_pool.search(cr, uid, [('drupal_user_id', '=', drupal_orders[order_id]['drupal_user_id'])]):
                user_id = openerp_drupal_users_pool.search(cr, uid, [('drupal_user_id', '=', drupal_orders[order_id]['drupal_user_id'])])[0]
            else:
                # Drupal User don't exist
                # AutoFill Fields
                print 'Cet User n\'existe pas'
                user = dpc.training.getUserDetails(drupal_orders[order_id]['drupal_user_id'], order_id)
                user_id = openerp_drupal_users_pool.create(cr, uid, {
                    'drupal_user_id': user['drupal_user_id'],
                    'drupal_email': user['drupal_email'],
                    'drupal_first_name': user['drupal_first_name'],
                    'drupal_last_name': user['drupal_last_name'],
                    'drupal_company': user['drupal_company'],
                    'drupal_street1': user['drupal_street1'],
                    'drupal_street2': user['drupal_street2'],
                    'drupal_city': user['drupal_city'],
                    'drupal_country': user['drupal_country'],
                    'drupal_state_province': user['drupal_state_province'],
                    'drupal_postal_code': user['drupal_postal_code'],
                    'drupal_phone_number': user['drupal_phone_number'],
                    'state': 'draft',
                })

                # Check if a contact's Job can correspond to this Drupal User
                contact_job_pool = self.pool.get('res.partner.job')
                contact_job_possible_ids = contact_job_pool.search(cr, uid, [('email', '=', user['drupal_email']), ('state', '=', 'current')])
                if contact_job_possible_ids:
                    w.trg_validate(uid, 'training.ubercart.webshop.drupal.users', user_id, 'action_confirm', cr)
                    openerp_drupal_users_pool.write(cr, uid, user_id, {
                        'job_id': contact_job_possible_ids[0],
                    })

            id = orders_pool.create(cr, uid, {
                'drupal_order_id': order_id,
                'drupal_user_id': user_id,
                'drupal_messages': '\r\n--\r\n'.join(drupal_orders[order_id]['comments']),
                'custom_ref': drupal_orders[order_id]['custom_reference'],
                }
            )

            # one2many subscription lines creation
            for dict in drupal_orders[order_id]['subscription_line']:
                sublines_pool.create(cr, uid, {
                    'session_id': dict['session_id'],
                    'exam_session_id': dict['exam_session_id'],
                    'order_id': id,
                })

            #subscription creation
            #drupal_user = openerp_drupal_users_pool.browse(cr, uid, user_id, context = context)

            #if drupal_user.state == 'validated':
            #    w.trg_validate(uid, 'training.ubercart.webshop.orders', id, 'signal_matching_confirmed', cr)
            #    try:
            #        cr.execute("SAVEPOINT wkf_drp_orders")
            #        try:
            #            w.trg_validate(uid, 'training.ubercart.webshop.orders', id, 'signal_confirm', cr)
            #            w.trg_validate(uid, 'training.ubercart.webshop.orders', id, 'signal_done', cr)
            #        except:
            #            cr.execute("ROLLBACK TO wkf_drp_orders")
            #    finally:
            #        cr.execute("RELEASE wkf_drp_orders")


training_ubercart_webshop_refresh_orders()
