# -*- coding: utf-8 -*-
##############################################################################
#
#    training_ubercart_webshop module for OpenERP, Fetch ubercart orders from IFBL Website
#    Copyright (C) 2011 Thamini SàRL (<http://http://www.thamini.lu) Jean-Christophe VASSORT
#
#    This file is a part of training_ubercart_webshop
#
#    training_ubercart_webshop is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    training_ubercart_webshop is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import netsvc

class training_ubercart_webshop_create_partner(osv.osv_memory):
    _name = "training.ubercart.webshop.create.partner"

    _columns = {
        'function_id': fields.many2one('res.partner.function', 'Function'),
        'state': fields.selection([('1', '1'),
                                   ('2', '2'),
                                  ('3', '3')]),
    }

    _defaults = {
        'state': lambda *a: '1'
    }

    def create_partner(self, cr, uid, ids, context=None):
        for o in self.pool.get(context['active_model']).browse(cr, uid, context['active_ids'], context=context):
            # define future partner name and Search if partner already exists
            future_partner_name = o.drupal_company and '%s %s (%s)' % (o.drupal_first_name, o.drupal_last_name.upper(), o.drupal_company) or '%s %s' % (o.drupal_first_name, o.drupal_last_name.upper())
            existing_partner_id = self.pool.get('res.partner').search(cr, uid, [('name', '=', future_partner_name)])
            if existing_partner_id:
                raise osv.except_osv(('Warning'),
                                      ("A partner is already existing with the same name. %(ids)s") % {'ids': existing_partner_id})
            contact_data = {
                'name': o.drupal_last_name,
                'first_name': o.drupal_first_name,
            }
            contact_id = self.pool.get('res.partner.contact').create(cr, uid, contact_data)

            customer_country_id = self.pool.get('res.country').search(cr, uid, [('name', '=', o.drupal_country)])[0]
            customer_function_id = self.read(cr, uid, ids, ['function_id'])[0]['function_id']

            partner_data = {
                'name': future_partner_name,
                'customer': True,
                'supplier': False,
                'address': [
                        (0, 0, {
                            'street': o.drupal_street1,
                            'street2': o.drupal_street2,
                            'zip': o.drupal_postal_code,
                            'city': o.drupal_city,
                            #'country_id': 129, # Luxembourg
                            'country_id': customer_country_id,
                            'type': 'default',
                            'phone': o.drupal_phone_number,
                            #'fax': 'sip:moneypenny@thamni.com',
                            'email': o.drupal_email,
                            'job_ids': [ (0, 0, {
                                    'function_id': customer_function_id,
                                    'email': o.drupal_email,
                                    'phone': o.drupal_phone_number,
                                    'date_start': '2011-06-08', # TODAY
                                    'state': 'current',
                                    'contact_id': contact_id,
                                    })],
                         })],
                #'notif_participant': True,
                #'notif_exam_participant': True,
            }

            # Create partner...
            partner_id = self.pool.get('res.partner').create(cr, uid, partner_data)
            #print("Partner id: %s" % (partner_id))

            # Search contact_id...
            contact_id = self.pool.get('res.partner.contact').search(cr, uid, [('partner_id', '=', partner_id)])[0]
            
            # ... to find job_id...
            job_id = self.pool.get('res.partner.job').search(cr, uid, [('contact_id', '=', contact_id)])[0]

            # add job_id to order
            self.pool.get(context['active_model']).write(cr, uid, context['active_ids'], {
                'job_id': job_id,
            })

            values = {
                'notif_contact_id': job_id,
                'notif_exam_contact_id': job_id,
            }

            self.pool.get('res.partner').write(cr, uid, [partner_id], values)

            w = netsvc.LocalService('workflow')

            if context['active_model'] == 'training.ubercart.webshop.drupal.users':
                drupal_user_id = context['active_ids'][0]
            else:
                drupal_user_id = o.drupal_user_id.id

            w.trg_validate(uid, 'training.ubercart.webshop.drupal.users', drupal_user_id, 'action_validate', cr)
            self.pool.get('training.ubercart.webshop.orders').action_update_all_draft_orders(cr, uid, ids, drupal_user_id, w)

            # Partner created and job added, update drupal user state

        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form,tree',
            'res_model': 'res.partner',
            'res_id': partner_id,
            'view_id': False,
            #'target': 'new',
        }

    def create_partner_ok(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': '2'})

training_ubercart_webshop_create_partner()
